from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
import undetected_chromedriver as uc
from selenium.webdriver.common.keys import Keys
from bs4 import BeautifulSoup
import time
import pandas as pd
import re  # 导入正则表达式模块
from datetime import datetime  # 导入日期时间模块

# 设置Selenium
options = webdriver.ChromeOptions()
options.add_argument('--ignore-certificate-errors') 
options.add_argument('--headless')  # 无头模式
options.add_argument('--log-level=3')
driver = uc.Chrome(executable_path='D:/anaconda/chromedriver.exe', options=options)


wait = WebDriverWait(driver, 30)
# base_url = 'https://hydramarket.org/Forums-database-logs-combolist?page={}&sortby=started'

# 设置超时
driver.set_page_load_timeout(10)  # 10秒
# 导航到登录页面
driver.get('https://10.20.240.31/accounts/login/')

# 等待用户名输入框出现
wait.until(EC.presence_of_element_located((By.ID, 'username')))

# 定位用户名和密码输入框，并输入登录凭证
username_input = driver.find_element(By.ID, 'username')
password_input = driver.find_element(By.ID, 'password')
username_input.send_keys('guotao')
password_input.send_keys('P@55Word')

# 定位并点击登录按钮
submit_button = driver.find_element(By.XPATH, '//input[@type="submit"]')
submit_button.click()

ILLEGAL_CHARACTERS_RE = re.compile(r'[\000-\010]|[\013-\014]|[\016-\037]')

def remove_illegal_chars(val):
    if isinstance(val, str):
        return ILLEGAL_CHARACTERS_RE.sub('', val)
    else:
        return val
        
def save_to_excel(data, batch_num):
    df = pd.DataFrame(data)
    # 应用remove_illegal_chars函数来清理每个单元格中的非法字符
    df = df.applymap(remove_illegal_chars)
    df.to_excel(f'output_batch_{batch_num}.xlsx', index=False)
    print(f"已保存批次 {batch_num} 到Excel文件。")

# # 添加适当的等待时间，确保登录操作完成
# wait.until(EC.presence_of_element_located((By.ID, 'ID_OF_ELEMENT_AFTER_LOGIN')))

# 读取ID
with open('output_ids.txt', 'r') as file:
    ids = [line.strip() for line in file]

# 创建一个空的Pandas DataFrame来保存数据
columns = ['漏洞名称', '漏洞描述', '解决方法', '危险分值', '危险插件', '发现日期']
df = pd.DataFrame(columns=columns)

count = 0  # 总计数
# 读取用户输入的批次号
start_batch_num = int(input("请输入要开始的批次号（从0开始）："))
batch_num = start_batch_num  # 设置当前批次号为用户输入的批次号
chunks = [ids[x:x+1000] for x in range(0, len(ids), 1000)]  # 分割ID成1000个一组


for chunk in chunks[start_batch_num:]:
    for id in chunk:
        if not id:
            print("ID为空，停止抓取。")
            break

        url = f"https://10.20.240.31/template/show_vul_desc?id={id}"

        try:
            driver.get(url)
        except Exception as e:
            print(f"无法加载URL: {url}，错误信息: {e}")
            continue  # 跳过当前迭代，继续下一个ID

        # 等待表格出现
        wait.until(EC.presence_of_element_located((By.CSS_SELECTOR, 'table.cmn_table.plumb')))

        # 获取表格HTML并用BeautifulSoup解析
        table_html = driver.find_element(By.CSS_SELECTOR, 'table.cmn_table.plumb').get_attribute('outerHTML')
        soup = BeautifulSoup(table_html, 'html.parser')
        table = soup.find('table')

        # 提取数据
        row_data = {}
        for row in table.find_all('tr'):
            header = row.find('th').text.strip()
            data = row.find('td').text.strip()
            row_data[header] = data

        # 添加到DataFrame
        df = df.append(row_data, ignore_index=True)
        count += 1
        print(f"第{batch_num+1}批次，已爬取{count}个ID。当前ID: {id}")
        # time.sleep(3)  # 可按需要调整暂停时间

    # 保存当前ID组的数据
    save_to_excel(df, batch_num)
    df = pd.DataFrame(columns=columns)  # 清空DataFrame以保存下一批次的数据
    batch_num += 1

    # 等待用户输入以开始下一批次
    input(f"批次 {batch_num} 已完成。按Enter键继续下一批次...")

print("爬取完毕。")
driver.quit()

# 等待元素加载并可点击
# 通过XPath查找链接
# https://10.20.240.31/template/show_vul_desc?id=75813
# link_element = wait.until(EC.element_to_be_clickable((By.XPATH, '//*[@id="40_category"]/span[3]/a')))
# https://10.20.240.31/template/getTreeHtml?val=System&conditions=is_dangerous=&temp_id=100&op_type=showUtemp&&checked=true&id=40_threatLevel_2&offset=0&limit=2

